﻿var ping = require('ping')
var moment = require('moment');

//---------------------------------------------
// Kenwood repeater control
//#region Kenwood manager - Initialize dB
var KenwoodControlRules = [];
// Initialize Kenwood Host array
exports.InitializeKenwoodManager = function (db, io, logger, callback) {

    // Try getting the Kenwood Repeater Control Rules table
    db.get("SELECT name FROM sqlite_master WHERE type='table' AND name='KenwoodControl'", function (err, rows) {
        if (err !== null) {
            logger.error('KenwoodManager.InitializeKenwoodManager: Initialize Error (Select)=' + JSON.stringify(err));
        }
        else if (rows === undefined) {
            // We read nothing, so let's create the table ...
            logger.info("KenwoodManager:InitializeKenwoodManager: Initialize by creating database table ...");
            db.exec("CREATE TABLE if not exists KenwoodControl (RepeaterName TEXT, Enabled TEXT, SendEvents TEXT, HostAddress TEXT, ToFail INTEGER, ToRecover INTEGER, ActivateOutput INTEGER)", function (err) {
                if (err !== null) {
                    logger.error('KenwoodManager.InitializeKenwoodManager: Initialize Error (Create)=' + JSON.stringify(err));
                }
                else {
                    // Send back empty array for newly created table ...
                    if (typeof callback == "function") {
                        logger.info('KenwoodManager.InitializeKenwoodManager: Initialized new database table');
                        callback(KenwoodControlRules);
                    }
                }
            });
        }
        else {
            // Table exists, so read from the database and populate the KenwoodControlRules array
            logger.info("KenwoodManager.InitializeKenwoodManager: Read Kenwood Control Rules from database ...");
            numberOfKenwoodControlRules = 0;
            KenwoodControlRules.length = 0;
            db.each("SELECT rowid AS Id, RepeaterName, Enabled, SendEvents, HostAddress, ToFail, ToRecover, ActivateOutput FROM KenwoodControl", function (err, row) {
                if (err !== null) {
                    logger.error('KenwoodManager.InitializeKenwoodManager: Initialize Error (Select)=' + JSON.stringify(err));
                } else {
                    var timeChanged = moment();// Record the create time as the last time the I/O point value was changed

                    // Copy each row into a KenwoodControlRules array
                    logger.trace('KenwoodManager.InitializeKenwoodManager: Processing Host[' + (numberOfKenwoodControlRules + 1) + ']=' + row.RepeaterName);
                    KenwoodControlRules.push({
                        Id              : row.Id,
                        RepeaterName    : row.RepeaterName,
                        Enabled         : row.Enabled,
                        SendEvents      : row.SendEvents,
                        HostAddress     : row.HostAddress,
                        ToFail          : row.ToFail,
                        ToRecover       : row.ToRecover,
                        ActivateOutput   : row.ActivateOutput,
                        TimeChanged     : timeChanged,
                        Status          : "UNKNOWN",
                        FailCount       : 0,
                        RecoverCount    : 0
                    });
                    logger.debug("KenwoodManager.InitializeKenwoodManager: Read Kenwood Rules from Database[" + numberOfKenwoodControlRules + "]=" + JSON.stringify(KenwoodControlRules[numberOfKenwoodControlRules]));
                    numberOfKenwoodControlRules++;
                }
            }, function () {
                // Retrurn the complete list of Kenwood Hosts
                logger.info('KenwoodManager.InitializeKenwoodManager: Initialized from existing database table');
                if (typeof callback == "function") {
                    callback(KenwoodControlRules);
                }
            });
        }
    });
}
//#endregion Kenwood manager - Initialize dB

//#region Kenwood manager - dB/array management
exports.GetKenwoodRules = function (logger, callback) {
    // Get Kenwood rules
    logger.debug('KenwoodManager.GetKenwoodRules');

    if (typeof callback == "function") {
        callback(KenwoodControlRules);
    }
}

exports.AddKenwoodRule = function (req, logger, db, callback) {
    // Add a new Kenwood rule record
    logger.debug('KenwoodManager.AddKenwoodRule: Kenwood rule to add=' + JSON.stringify(req.body));
    
    // Insert the new record in to the database
    insertStatement = "'" + req.body.RepeaterName + "', '" + req.body.Enabled + "', '" + req.body.SendEvents + "', '" + req.body.HostAddress + "', " + req.body.ToFail + ", " + req.body.ToRecover + ", " + req.body.ActivateOutput;
    logger.debug('KenwoodManager.AddKenwoodRule: dB insert string=' + insertStatement);
    db.run("BEGIN")
    db.run("INSERT INTO KenwoodControl (RepeaterName, Enabled, SendEvents, HostAddress, ToFail, ToRecover, ActivateOutput) VALUES (" + insertStatement + ")", function (err) {
        if (err !== null) {
            logger.error('KenwoodManager.AddKenwoodRule: Error (Insert)=' + err);
            
            // Return the Kenwood rules, which now includes new element
            if (typeof callback == "function") {
                callback({ msg: "SQLite3 dB: api/kenwood/rules Insert err=" + JSON.stringify(err) });
            }
        }
        
        // Successful insert, just look up the ID for future reference
        req.body.Id = this.lastID;
        logger.debug('KenwoodManager.AddKenwoodRule: Added rule @ ID=' + req.body.Id);
        
        // Append new rule to the existing array
        var timeChanged = moment();
        KenwoodControlRules.push({
            Id              : req.body.Id,
            RepeaterName    : req.body.RepeaterName,
            Enabled         : req.body.Enabled,
            SendEvents      : req.body.SendEvents,
            HostAddress     : req.body.HostAddress,
            ToFail          : req.body.ToFail,
            ToRecover       : req.body.ToRecover,
            ActivateOutput  : req.body.ActivateOutput,
            TimeChanged     : timeChanged,
            Status          : "UNKNOWN",
            FailCount       : 0,
            RecoverCount    : 0
        });

        // Return the Kenwood rules, which now includes new element
        if (typeof callback == "function") {
            callback(KenwoodControlRules);
        }
    });
    db.run("COMMIT");
}

exports.DeleteKenwoodRule = function (req, logger, db, callback) {
    // Delete a Kenwood rule record
    logger.debug('KenwoodManager.DeleteKenwoodRule: Kenwood rule to add=' + JSON.stringify(req.body));

    var update = "DELETE FROM KenwoodControl WHERE rowid='" + req.body.Id + "'";
    db.run(update, function (err) {
        if (err !== null) {
            logger.error('KenwoodManager.DeleteKenwoodRule:  Error (Update)=' + JSON.stringify(err));
            
            if (typeof callback == "function") {
                callback(KenwoodControlRules);
            }
            return;
        }

        // Successful insert, just look up the ID for future reference
        logger.debug('KenwoodManager.DeleteKenwoodRule:: Deleted rule @ ID=' + req.body.Id);
        
        // Delete the rule from the existing array
        for (i = 0; i < KenwoodControlRules.length; i++) {
            if (KenwoodControlRules[i].Id == req.body.Id) {
                KenwoodControlRules.splice(i, 1);// remove the item that we want to delete
                break;
            }
        }

        // Return the Kenwood rules, which now has removed the deleted element
        if (typeof callback == "function") {
            callback(KenwoodControlRules);
        }        
    });
}

exports.ModifyKenwoodRule = function (req, logger, io, db, netSnmp, callback) {
    // Delete a Kenwood rule record
    logger.debug('KenwoodManager.ModifyKenwoodRule: Kenwood rule to modify=' + JSON.stringify(req.body));
    
    // Update the existing record
    var update = "UPDATE KenwoodControl SET RepeaterName='" + req.body.RepeaterName + "', Enabled='" + req.body.Enabled + "', SendEvents='" + req.body.SendEvents + "', HostAddress='" + req.body.HostAddress + "', ToFail=" + req.body.ToFail + ", ToRecover=" + req.body.ToRecover + ", ActivateOutput=" + req.body.ActivateOutput + " WHERE rowid='" + req.params.kenwoodrule + "'";
    db.run(update, function (err) {
        if (err !== null) {
            logger.error('KenwoodManager.ModifyKenwoodRule: ' + req.params.kenwoodrule + ' - Error (Update)=' + JSON.stringify(err));
            
            if (typeof callback == "function") {
                callback(KenwoodControlRules);
            }
            return;
        }
        
        // Update the existing rule
        for (i = 0; i < KenwoodControlRules.length; i++) {
            if (KenwoodControlRules[i].Id == req.body.Id) {
                KenwoodControlRules[i].RepeaterName = req.body.RepeaterName;
                KenwoodControlRules[i].ToFail = req.body.ToFail;
                KenwoodControlRules[i].ToRecover = req.body.ToRecover;
                KenwoodControlRules[i].ActivateOutput = req.body.ActivateOutput;
                
                // Check to see if IP address or Enabled changed changed, if it did, then restart ping campaign                
                if ((KenwoodControlRules[i].HostAddress != req.body.HostAddress) || (KenwoodControlRules[i].Enabled != req.body.Enabled)) {
                    KenwoodControlRules[i].Status = "UNKNOWN";
                    KenwoodControlRules[i].TimeChanged = moment();
                    KenwoodControlRules[i].Enabled = req.body.Enabled;
                    KenwoodControlRules[i].SendEvents = req.body.SendEvents;
                    KenwoodControlRules[i].FailCount = 0;
                    KenwoodControlRules[i].RecoverCount = 0;
                    KenwoodControlRules[i].HostAddress = req.body.HostAddress;
                    
                    // Send an update because this element has changed!
                    sendSocketAndSnmp(KenwoodControlRules[i], db, io, logger, netSnmp)
                }
                KenwoodControlRules[i].HostAddress = req.body.HostAddress;
                // Return the Kenwood rules, which now has removed the modified element
                if (typeof callback == "function") {
                    callback(KenwoodControlRules[i]);
                }
                break;
            }
        }        
    }); 
}
//#endregion Kenwood manager - dB/array management

//#region Kenwood Manager - Process pings and send results
exports.ProcessKenwoodPings = function (db, logger, io, netSnmp, ledManager, digitalOutputs, callback) {
    var timeChanged = moment();
    for (index = 0; index < KenwoodControlRules.length; index++) {
        //logger.info('KenwoodManager.ProcessKenwoodPings: Host=' + KenwoodControlRules[index].HostAddress + " Enabled=" + KenwoodControlRules[index].Enabled);
        
        // Only ping the host if the rule is enabled
        if (KenwoodControlRules[index].Enabled === "TRUE") {
            try {
                // Ping the host
                KenwoodHostPing(logger, index, function (index, status) {
                    var msg = status ? 'ALIVE' : 'DEAD';
                    
                    // Depending on current state, check if state changed
                    switch (KenwoodControlRules[index].Status) {
                        case "UNKNOWN":// Start-up state
                            if (status) {
                                // Successful ping
                                KenwoodControlRules[index].RecoverCount++;
                                KenwoodControlRules[index].FailCount = 0;
                                
                                if (KenwoodControlRules[index].RecoverCount >= KenwoodControlRules[index].ToRecover) {
                                    // UKNOWN -> ALIVE !!!
                                    KenwoodControlRules[index].Status = "ALIVE";
                                    KenwoodControlRules[index].TimeChanged = moment();
                                    sendSocketAndSnmp(KenwoodControlRules[index], db, io, logger, netSnmp);
                                    digitalOutputs.SetDigitalOutput((parseInt(KenwoodControlRules[index].ActivateOutput) - 1), 0, logger, db, function (result) {
                                        logger.info('KenwoodManager.ProcessKenwoodPings: State Change (UNKNOWN->ALIVE) DigitalOutput #' + KenwoodControlRules[index].ActivateOutput + ' Value=1  Result=' + result);
                                    });
                                }
                            } else {
                                // Failed ping
                                KenwoodControlRules[index].FailCount++;
                                KenwoodControlRules[index].RecoverCount = 0;

                                if (KenwoodControlRules[index].FailCount >= KenwoodControlRules[index].ToFail) {
                                    // UNKNOWN -> DEAD !!!
                                    KenwoodControlRules[index].Status = "DEAD";
                                    KenwoodControlRules[index].TimeChanged = moment();
                                    sendSocketAndSnmp(KenwoodControlRules[index], db, io, logger, netSnmp);
                                    digitalOutputs.SetDigitalOutput((parseInt(KenwoodControlRules[index].ActivateOutput) - 1), 1, logger, db, function (result) {
                                        logger.info('KenwoodManager.ProcessKenwoodPings: State Change (UNKNOWN->DEAD) DigitalOutput #' + KenwoodControlRules[index].ActivateOutput + ' Value=1  Result=' + result);
                                    });
                                }
                            }
                            break;

                        case "DEAD":// Broken connection
                            KenwoodControlRules[index].FailCount = KenwoodControlRules[index].ToFail;// Set to maximum
                            if (status) {
                                // Successful ping
                                KenwoodControlRules[index].RecoverCount++;
                                KenwoodControlRules[index].FailCount = 0;

                                if (KenwoodControlRules[index].RecoverCount >= KenwoodControlRules[index].ToRecover) {
                                    // DEAD -> ALIVE !!!
                                    KenwoodControlRules[index].Status = "ALIVE";
                                    KenwoodControlRules[index].TimeChanged = moment();
                                    sendSocketAndSnmp(KenwoodControlRules[index], db, io, logger, netSnmp);
                                    digitalOutputs.SetDigitalOutput((parseInt(KenwoodControlRules[index].ActivateOutput) - 1), 0, logger, db, function (result) {
                                        logger.info('KenwoodManager.ProcessKenwoodPings: State Change (DEAD->ALIVE) DigitalOutput #' + KenwoodControlRules[index].ActivateOutput + ' Value=1  Result=' + result);
                                    });
                                }
                            } else {
                                // Failed ping
                                KenwoodControlRules[index].RecoverCount = 0;// Reset, we're still failing
                            }
                            break;

                        case "ALIVE":// Good connection
                            KenwoodControlRules[index].RecoverCount = KenwoodControlRules[index].ToRecover;// Set to maximum
                            if (!status) {
                                // Failed ping
                                KenwoodControlRules[index].FailCount++;
                                KenwoodControlRules[index].RecoverCount = 0;
                                
                                if (KenwoodControlRules[index].FailCount >= KenwoodControlRules[index].ToFail) {
                                    // ALIVE -> DEAD !!!
                                    KenwoodControlRules[index].Status = "DEAD";
                                    KenwoodControlRules[index].TimeChanged = moment();
                                    sendSocketAndSnmp(KenwoodControlRules[index], db, io, logger, netSnmp);
                                    digitalOutputs.SetDigitalOutput((parseInt(KenwoodControlRules[index].ActivateOutput) - 1), 1, logger, db, function (result) {
                                        logger.info('KenwoodManager.ProcessKenwoodPings: State Change (ALIVE->DEAD) DigitalOutput #' + KenwoodControlRules[index].ActivateOutput + ' Value=1  Result=' + result);
                                    });
                                }
                            } else {
                                // Successful ping
                                KenwoodControlRules[index].FailCount = 0;// Reset, we're still succeeding
                            }
                            break;
                    }
                    //if (msg == 'Dead')
                    logger.info('KenwoodManager.ProcessKenwoodPings: Host=' + KenwoodControlRules[index].HostAddress + ' ping completed, result=' + msg + ", FailCount=" + KenwoodControlRules[index].FailCount + ", RecoverCount=" + KenwoodControlRules[index].RecoverCount);
                });
                
                // Now check if it's been passed the hold time, if so, affect change
                //if (KenwoodControlRules[index].ValueChanged == true) {
                    
                //    // Send instanteous changes to UI
                //    sendKenwoodRuleStatusChange();
                    
                //    // Send SNMP if (a) The hold time has been met and (b) we are at a value different than when we started the hold time
                //    sendSocketAndSnmp(KenwoodControlRules[index], db, io, logger, netSnmp)
                    
                //    // Update the status in the database
                //}
            }
       		catch (error) {
                logger.error('KenwoodManager.ProcessKenwoodPings - Process Error (Read)=' + JSON.stringify(error));
                
                if (typeof callback == "function") {
                    callback(KenwoodControlRules);
                }
            }
        } else {
            logger.info('KenwoodManager.ProcessKenwoodPings - Kenwood Rule is ignored for: '+ KenwoodControlRules[index].HostAddress);        
        }
    }
   
    if (typeof callback == "function") {
        callback(KenwoodControlRules);
    }
}

function KenwoodHostPing(logger, index, callback){
    ping.sys.probe(KenwoodControlRules[index].HostAddress, function (isAlive) {
        if (typeof callback == "function") {
            callback(index, isAlive);
        }
    });
}

// Send updates to clients
function sendSocketAndSnmp(kenwoodRule, db, io, logger, netSnmp) {
    logger.info("KenwoodManager.sendSocketAndSnmp: Rule - [Value UPDATED] KenwoodControlRules[" + kenwoodRule.Id + "] '" + kenwoodRule.RepeaterName + "' Value=" + kenwoodRule.Status);
    logger.debug("KenwoodManager.sendSocketAndSnmp: Rule - [Value UPDATED] KenwoodControlRules[" + kenwoodRule.Id + "] =" + JSON.stringify(kenwoodRule));
    
    // SOCKET.IO: Send changed value to all attached clients
    sendValueKenwoodRule("KenwoodRule", kenwoodRule, io, logger);
    
    // SNMP: Send SNMP Trap/Notification if threshold(s) crossed
    //sendSNMP(digitalInput, io, logger, netSnmp);
}

function sendValueKenwoodRule(dataType, data, io, logger) {
    io.sockets.emit(dataType, data);
}
//#endregion Kenwood Manager - Process pings and send results