﻿// Dependencies and Globals
//#region Dependencies and Globals
var rc4a = require('rc4a');
var moment = require('moment');
var fs = require('fs');
var cp = require('child_process');

var SummitPassPhrase = "TotallyAwesomeSuperCool!";// This is our super duper secure phrase
var SummitUser = {};
var SummitLoginAttempts = [];
var SUMMIT_LOGIN_ATTEMPTS = 10;

var Logger;// Logging global

//#endregion Dependencies and Globals

// Initialize
//#region Initialize
exports.Initialize = function (logger) {
    Logger = logger;// Globalize
    
    try {
        // Try to read the users file ...
        SummitUser = require("./users.json");
    }
    catch (error) {
        // Something went wrong, so get golden copy from ./default directory
        //  This will also happen on a fresh install of crest because users.json doesn't exist,
        //  and after a factory reset
        Logger.error("Users.LoginUsers: Unable to find user accounts, loading factory default user accounts. Error=" + error);
        cp.exec("cp ./factory/users.json ./users.json", function (error, stdout, stderr) {
            Logger.trace("Users.LoginUsers: ./factory/users.json ./users.json - success=" + stdout);
            if (error !== null) {
                Logger.error("Users.LoginUsers: Error in recovering factory default user accounts. Error=" + error);
            } else {
                // Sync the results so the files are made persistent
                cp.exec("sync", function (error, stdout, stderr) {
                    if (error !== null) {
                        Logger.error("Users.LoginUsers: Error in sync after loading factory default user accounts. Error=" + error);
                    } else {
                        SummitUser = require("./users.json");
                        Logger.info("Users.LoginUsers: Successfully recovered factory default user accounts.");
                    }
                });                
            }
        });
    }
}
//#endregion Initialize

// Get Summit User
//#region Get Summit User
exports.GetSummitUser = function (callback) {
    // For now, we'll always send decrypted password, so it's easier for client
    //  *** FUTURE: send encrypted password, because client will be able to manage RC4A
    var summitUser = JSON.parse(JSON.stringify(SummitUser));
    summitUser.password = rc4a.decrypt(SummitUser.password, SummitPassPhrase);
    callback(summitUser);
}
//#endregion Get Summit User

// Put Summit User
//#region Get Summit User
exports.PutSummitUser = function (summitUser, callback) {
    // For now, we'll always get decrypted password, so it's easier for client
    //  Therefore, we must encrypt before saving
    
    // Update SummitUser
    SummitUser.user = summitUser.user;
    SummitUser.password = rc4a.encrypt(summitUser.password, SummitPassPhrase);
    SummitUser.lastlogin = moment();
    summitUser.lastlogin = SummitUser.lastlogin;
    
    // Update users.json with new SummitUser settings to ensure persistence
    fs.writeFile('users.json', JSON.stringify(SummitUser), function (error) {
        if (error) {
            Logger.error("users.PutSummitUser: User="+summitUser.user+" account update write error. Error=" + error);
        }
        Logger.info("users.PutSummitUser: User="+summitUser.user+" account has been udpated");
        callback(summitUser);
    });
}
//#endregion Put Summit User

// GetSummitLoginAttempts
//#region GetSummitLoginAttempts
//  callback    - callback: SummitLoginAttempts
//#region Login Attempts
exports.GetSummitLoginAttempts = function (callback) {
    callback(SummitLoginAttempts);
}
//#endregion GetSummitLoginAttempts

// Login User
//  login       - login credentials
//  ip          - IP address of client
//  callback    - callback: success=true, failure=false
//#region Login User
exports.LoginUser = function (login, ip, callback) {
    //Logger.info("Users.LoginUsers: Entered=" + JSON.stringify(login) + " Actual=" + JSON.stringify(SummitUser));
    //Logger.info("Users.LoginUsers: Entered password=" + login.password + " Actual password=" + rc4a.decrypt(SummitUser.password, SummitPassPhrase));
    
    var success = false;
    var loginTime = moment();
    if ((login.user === SummitUser.user) && (login.password === rc4a.decrypt(SummitUser.password,SummitPassPhrase))) {
        
        // Login was successful, save the login time
        SummitUser.lastlogin = loginTime;
        
        // Save the users file with new login time
        // Update ports.json to ensure persistence
        fs.writeFile('users.json', JSON.stringify(SummitUser), function (error) {
            if (error) {
                Logger.error("Users.LoginUsers: Error writing users account after successful login. Error=" + error);
                success = false;
            } else {
                Logger.info("Users.LoginUsers: Successful login - " + SummitUser.user + " (from:" + ip + ") has successfully logged in at " + SummitUser.lastlogin);
                success = true;
            }
            SummitLoginAttempts.unshift({ "time": loginTime, "status": "Successful Login", "user": login.user, "ip": ip });
            if (SummitLoginAttempts.length > SUMMIT_LOGIN_ATTEMPTS) SummitLoginAttempts.splice(-1);
            callback(success);
        });
    }
    else {
        // Credentials do not match ...
        Logger.error("Users.LoginUsers: Failed login by" + SummitUser.user + " (from:" + ip + ") at " + loginTime);
        SummitLoginAttempts.unshift({ "time": loginTime, "status": "Credentials did not match", "user": login.user, "ip": ip });
        if (SummitLoginAttempts.length > SUMMIT_LOGIN_ATTEMPTS) SummitLoginAttempts.splice(-1);
        callback(success);
    }
}
//#endregion Login User